--- src/resolver.c.orig	2011-04-26 10:08:17.000000000 +0200
+++ src/resolver.c	2011-04-26 11:36:53.000000000 +0200
@@ -124,21 +124,30 @@
 #endif
     hints.ai_socktype=SOCK_STREAM;
     hints.ai_protocol=IPPROTO_TCP;
-
-    err=getaddrinfo(hostname, portname, &hints, &res);
-    if(err) {
-        if(err==EAI_SERVICE)
-            s_log(LOG_ERR, "Unknown TCP service '%s'", portname);
-        else
-            s_log(LOG_ERR, "Error resolving '%s': %s",
-                hostname, s_gai_strerror(err));
-        if(res)
+    do {
+        err=getaddrinfo(hostname, portname, &hints, &res);
+        if(err && res)
             freeaddrinfo(res);
+        if(err==EAI_AGAIN) {
+            s_log(LOG_DEBUG, "getaddrinfo: EAI_AGAIN received: retrying");
+            sleep(1);
+        }
+    } while(err==EAI_AGAIN);
+    switch(err) {
+    case 0:
+        break; /* success */
+    case EAI_SERVICE:
+        s_log(LOG_ERR, "Unknown TCP service '%s'", portname);
+        return 0; /* error */
+    default:
+        s_log(LOG_ERR, "Error resolving '%s': %s",
+            hostname, s_gai_strerror(err));
         return 0; /* error */
     }
+
     /* copy the list of addresses */
-    cur=res;
-    while(cur && addr_list->num<MAX_HOSTS) {
+    for(cur=res; cur && addr_list->num<MAX_HOSTS;
+            cur=cur->ai_next, addr_list->num++) {
         if(cur->ai_addrlen>sizeof(SOCKADDR_UNION)) {
             s_log(LOG_ERR, "INTERNAL ERROR: ai_addrlen value too big");
             freeaddrinfo(res);
@@ -146,8 +155,6 @@
         }
         memcpy(&addr_list->addr[addr_list->num],
             cur->ai_addr, cur->ai_addrlen);
-        cur=cur->ai_next;
-        addr_list->num++;
     }
     freeaddrinfo(res);
     return addr_list->num; /* ok - return the number of addresses */
